@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -200,7 +200,13 @@ def _getAnonBuiltins():
 
     if python_version >= 0x270:
         anon_names["version_info"] = type(sys.version_info)
-        anon_codes["instancemethod"] = 'Py_TYPE(Nuitka_SysGetObject("version_info"))'
+        anon_codes["version_info"] = 'Py_TYPE(Nuitka_SysGetObject("version_info"))'
+
+    if python_version >= 0x3A0:
+        # 3.10 only code, pylint: disable=I0021,unsupported-binary-operation
+
+        anon_names["UnionType"] = type(int | str)
+        anon_codes["UnionType"] = "Nuitka_PyUnion_Type"
 
     return anon_names, anon_codes
 
