@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,6 +33,7 @@ from nuitka.PythonVersions import (
     python_version_str,
 )
 from nuitka.Tracing import postprocessing_logger
+from nuitka.utils.Execution import wrapCommandForDebuggerForExec
 from nuitka.utils.FileOperations import (
     getExternalUsePath,
     getFileContents,
@@ -217,9 +218,6 @@ def _addWindowsIconFromIcons(onefile):
         )
 
 
-version_resources = {}
-
-
 def executePostProcessingResources(manifest, onefile):
     """Adding Windows resources to the binary.
 
@@ -243,16 +241,15 @@ def executePostProcessingResources(manifest, onefile):
         or Options.getWindowsProductVersion()
         or Options.getWindowsFileVersion()
     ):
-        version_resources.update(
-            addVersionInfoResource(
-                string_values=Options.getWindowsVersionInfoStrings(),
-                product_version=Options.getWindowsProductVersion(),
-                file_version=Options.getWindowsFileVersion(),
-                file_date=(0, 0),
-                is_exe=not Options.shallMakeModule(),
-                result_filename=result_filename,
-                logger=postprocessing_logger,
-            )
+
+        addVersionInfoResource(
+            string_values=Options.getWindowsVersionInfoStrings(),
+            product_version=Options.getWindowsProductVersion(),
+            file_version=Options.getWindowsFileVersion(),
+            file_date=(0, 0),
+            is_exe=not Options.shallMakeModule(),
+            result_filename=result_filename,
+            logger=postprocessing_logger,
         )
 
     # Attach icons from template file if given.
@@ -353,10 +350,11 @@ def executePostProcessing():
                     os.path.join(python_lib_path, python_dll_filename),
                 ),
             ),
+            id_path=None,
             rpath=python_lib_path,
         )
 
-    if isMacOS() and Options.shallCreateAppBundle():
+    if Options.shallCreateAppBundle():
         createPlistInfoFile(logger=postprocessing_logger, onefile=False)
 
     # Modules should not be executable, but Scons creates them like it, fix
@@ -373,7 +371,8 @@ def executePostProcessing():
         if os.path.exists(candidate):
             os.unlink(candidate)
 
-    if isWin32Windows() and Options.shallTreatUninstalledPython():
+    # Might have to create a CMD file, potentially with debugger run.
+    if Options.shallCreateCmdFileForExecution():
         dll_directory = getExternalUsePath(os.path.dirname(getTargetPythonDLLPath()))
 
         cmd_filename = OutputDirectories.getResultRunFilename(onefile=False)
@@ -383,8 +382,11 @@ def executePostProcessing():
 rem This script was created by Nuitka to execute '%(exe_filename)s' with Python DLL being found.
 set PATH=%(dll_directory)s;%%PATH%%
 set PYTHONHOME=%(dll_directory)s
-"%%~dp0.\\%(exe_filename)s" %%*
+%(debugger_call)s"%%~dp0.\\%(exe_filename)s" %%*
 """ % {
+            "debugger_call": (" ".join(wrapCommandForDebuggerForExec()) + " ")
+            if Options.shallRunInDebugger()
+            else "",
             "dll_directory": dll_directory,
             "exe_filename": os.path.basename(result_filename),
         }
@@ -393,7 +395,7 @@ set PYTHONHOME=%(dll_directory)s
 
     # Create a ".pyi" file for created modules
     if Options.shallMakeModule() and Options.shallCreatePyiFile():
-        pyi_filename = OutputDirectories.getResultBasepath() + ".pyi"
+        pyi_filename = OutputDirectories.getResultBasePath() + ".pyi"
 
         putTextFileContents(
             filename=pyi_filename,
