@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,7 +25,6 @@ to add to this and submit patches to make it more complete.
 import fnmatch
 
 from nuitka.__past__ import iter_modules
-from nuitka.containers.oset import OrderedSet
 from nuitka.Options import (
     isOnefileMode,
     isStandaloneMode,
@@ -37,21 +36,14 @@ from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.Utils import getOS, isMacOS, isWin32Windows
-from nuitka.utils.Yaml import parsePackageYaml
+from nuitka.utils.Yaml import getYamlPackageConfiguration
 
 
 class NuitkaPluginImplicitImports(NuitkaPluginBase):
     plugin_name = "implicit-imports"
 
     def __init__(self):
-        self.config = parsePackageYaml(__package__, "implicit-imports.yml")
-
-        for key in self.config.keys():
-            if "/" in key:
-                self.sysexit(
-                    "Error, invalid key in 'implicit-imports.yml' looks like a file path, not module name '%s'."
-                    % key
-                )
+        self.config = getYamlPackageConfiguration()
 
     @staticmethod
     def isAlwaysEnabled():
@@ -68,10 +60,11 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
                     "Error, invalid pattern with empty parts used '%s'." % pattern
                 )
 
-            if "." in part or "*" in part:
+            # TODO: Checking for shell pattern should be done in more places and shared code.
+            if "?" in part or "*" in part or "[" in part:
                 if current is None:
                     self.sysexit(
-                        "Error, cannot use patter for first part '%s'." % pattern
+                        "Error, cannot use pattern for first part '%s'." % pattern
                     )
 
                 module_filename = self.locateModule(
@@ -101,17 +94,11 @@ class NuitkaPluginImplicitImports(NuitkaPluginBase):
 
         yield current
 
-    def _getImportsByFullname(self, full_name):
-        """Provides names of modules to imported implicitly.
-
-        Notes:
-            This methods works much like 'getImplicitImports', except that it
-            accepts the search argument as a string. This allows callers to
-            obtain results, which cannot provide a Nuitka module object.
-        """
+    def _getImportsByFullname(self, module, full_name):
+        """Provides names of modules to imported implicitly."""
         # Many variables, branches, due to the many cases, pylint: disable=too-many-branches,too-many-statements
 
-        config = self.config.get(full_name)
+        config = self.config.get(full_name, section="implicit-imports")
 
         # Checking for config, but also allowing fall through.
         if config:
@@ -172,10 +159,16 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
                 if (
                     disable_console == "recommend"
                     and mayDisableConsoleWindow()
-                    and not shallDisableConsoleWindow()
+                    and shallDisableConsoleWindow() is None
                 ):
+                    if isMacOS():
+                        downside_message = "Otherwise high resolution will not be available and a terminal window will open."
+                    else:
+                        downside_message = "Otherwise a terminal window will open."
+
                     self.info(
-                        "Note, when using '%s', consider using '--disable-console' option."
+                        "Note, when using '%s', consider using '--disable-console' option. %s"
+                        % (full_name, downside_message)
                     )
 
             if (
@@ -191,7 +184,15 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
 
             for dependency in dependencies:
                 if dependency.startswith("."):
-                    dependency = full_name.getChildNamed(dependency[1:]).asString()
+                    if (
+                        module.isUncompiledPythonPackage()
+                        or module.isCompiledPythonPackage()
+                    ):
+                        dependency = full_name.getChildNamed(dependency[1:]).asString()
+                    else:
+                        dependency = full_name.getSiblingNamed(
+                            dependency[1:]
+                        ).asString()
 
                 if "*" in dependency or "?" in dependency:
                     for resolved in self._resolveModulePattern(dependency):
@@ -780,7 +781,7 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
             yield "pywt._extensions._swt"
 
         # imageio imports -----------------------------------------------
-        elif full_name == "imageio":
+        elif full_name == "PIL.Image":
             yield "PIL.BlpImagePlugin"
             yield "PIL.BmpImagePlugin"
             yield "PIL.BufrStubImagePlugin"
@@ -1157,6 +1158,8 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
                 yield crypto_module_name + ".PublicKey._ec_ws"
             elif full_name == crypto_module_name + ".Cipher.ARC4":
                 yield crypto_module_name + ".Cipher._ARC4"
+            elif full_name == crypto_module_name + ".Cipher.PKCS1_v1_5":
+                yield crypto_module_name + ".Cipher._pkcs1_decode"
             elif full_name == crypto_module_name + ".Math._IntegerCustom":
                 yield crypto_module_name + ".Math._modexp"
         elif full_name == "pycparser.c_parser":
@@ -1197,6 +1200,7 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
         elif full_name == "pyreadstat.pyreadstat":
             yield "pyreadstat._readstat_writer"
             yield "pyreadstat.worker"
+            yield "multiprocessing"
         elif full_name == "cytoolz.itertoolz":
             yield "cytoolz.utils"
         elif full_name == "cytoolz.functoolz":
@@ -1227,33 +1231,10 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
             yield "mercurial.charencode"
             yield "mercurial.cext.parsers"
 
-    def getImportsByFullname(self, full_name):
-        """Recursively create a set of imports for a fullname.
-
-        Notes:
-            If an imported item has imported kids, call again with each new item,
-            resulting in a leaf-only set (no more consequential kids).
-        """
-        result = OrderedSet()
-
-        def checkImportsRecursive(module_name):
-            for item in self._getImportsByFullname(module_name):
-                item = ModuleName(item)
-
-                if item not in result:
-                    result.add(item)
-                    checkImportsRecursive(item)
-
-        checkImportsRecursive(full_name)
-
-        if full_name in result:
-            result.remove(full_name)
-
-        return result
-
     def getImplicitImports(self, module):
         full_name = module.getFullName()
 
+        # TODO: This code absolutely doesn't below here.
         if module.isPythonExtensionModule():
             for used_module in module.getUsedModules():
                 yield used_module[0]
@@ -1270,12 +1251,12 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
                 "jaraco",
                 "importlib_resources",
                 "more_itertools",
+                "six",
             ):
                 yield "pkg_resources._vendor." + part
 
         else:
-            # create a flattened import set for full_name and yield from it
-            for item in self.getImportsByFullname(full_name):
+            for item in self._getImportsByFullname(module=module, full_name=full_name):
                 yield item
 
     def onModuleSourceCode(self, module_name, source_code):
@@ -1325,6 +1306,7 @@ Error, package '%s' requires '--onefile' to be used on top of '--macos-create-ap
         "site",  # Not performance relevant and problems with .pth files
         "packaging",  # Not performance relevant.
         "appdirs",  # Not performance relevant.
+        "dropbox.team_log",  # Too large generated code
     )
 
     def decideCompilation(self, module_name):
