@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,12 +24,8 @@ own dependencies.
 import os
 
 from nuitka.containers.oset import OrderedSet
-from nuitka.freezer.IncludedDataFiles import (
-    makeIncludedDataFile,
-    makeIncludedGeneratedDataFile,
-)
 from nuitka.freezer.IncludedEntryPoints import makeExeEntryPoint
-from nuitka.Options import isStandaloneMode
+from nuitka.Options import isExperimental, isStandaloneMode
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.plugins.Plugins import getActiveQtPlugin
 from nuitka.PythonVersions import python_version
@@ -53,12 +49,14 @@ class NuitkaPluginQtBindingsPluginBase(NuitkaPluginBase):
     # For overload in the derived bindings plugin.
     binding_name = None
 
+    warned_about = set()
+
     def __init__(self, qt_plugins, no_qt_translations):
         self.qt_plugins = OrderedSet(x.strip().lower() for x in qt_plugins.split(","))
         self.no_qt_translations = no_qt_translations
 
-        self.webengine_done_binaries = False
-        self.webengine_done_data = False
+        self.web_engine_done_binaries = False
+        self.web_engine_done_data = False
         self.qt_plugins_dirs = None
 
         self.binding_package_name = ModuleName(self.binding_name)
@@ -77,12 +75,10 @@ class NuitkaPluginQtBindingsPluginBase(NuitkaPluginBase):
 
         if active_qt_plugin_name is not None:
             self.sysexit(
-                "Error, confliciting plugin '%s', you can only have one enabled."
+                "Error, conflicting plugin '%s', you can only have one enabled."
                 % active_qt_plugin_name
             )
 
-        self.warned_about = set()
-
     @classmethod
     def addPluginCommandLineOptions(cls, group):
         group.add_option(
@@ -267,7 +263,9 @@ import %(binding_name)s.QtCore
         )
 
         if info is None:
-            self.sysexit("Error, it seems '%s' is not installed." % self.binding_name)
+            self.sysexit(
+                "Error, it seems '%s' is not installed or broken." % self.binding_name
+            )
 
         return info
 
@@ -284,7 +282,7 @@ import %(binding_name)s.QtCore
         return self._getQtInformation().translations_path
 
     def _getResourcesPath(self):
-        """Get the path to the Qt webengine resources."""
+        """Get the path to the Qt web engine resources."""
         return os.path.join(self._getQtInformation().data_path, "resources")
 
     def _getLibraryExecutablePath(self):
@@ -346,7 +344,7 @@ import %(binding_name)s.QtCore
     def _getQmlFileList(self, dlls):
         qml_plugin_dir = self._getQmlDirectory()
 
-        # List all file types of the QML plugin folder that are datafiles and not DLLs.
+        # List all file types of the QML plugin folder that are data files and not DLLs.
         datafile_suffixes = (
             ".qml",
             ".qmlc",
@@ -384,6 +382,15 @@ import %(binding_name)s.QtCore
                 if not self.hasQtPluginSelected(qt_plugin_name):
                     continue
 
+                # TODO: The qpdf plugin is causing issues, and we would have to check
+                # here, already, if dependencies of the plugin will be available, and
+                # if not, then skip it, but that is for a future release, as it is
+                # blocking on macOS now.
+                if "qpdf" in os.path.basename(filename_relative) and not isExperimental(
+                    "qt-force-qpdf"
+                ):
+                    continue
+
                 yield self.makeDllEntryPoint(
                     source_path=filename,
                     dest_path=os.path.join(
@@ -643,20 +650,21 @@ if not path.startswith(__nuitka_binary_dir):
             for filename in self._getQmlFileList(dlls=False):
                 filename_relative = os.path.relpath(filename, qml_plugin_dir)
 
-                yield makeIncludedDataFile(
+                yield self.makeIncludedDataFile(
                     source_path=filename,
                     dest_path=os.path.join(
                         qml_target_dir,
                         filename_relative,
                     ),
                     reason="Qt QML datafile",
+                    tags="qml",
                 )
-        elif self.isQtWebEngineModule(full_name) and not self.webengine_done_data:
-            self.webengine_done_data = True
+        elif self.isQtWebEngineModule(full_name) and not self.web_engine_done_data:
+            self.web_engine_done_data = True
 
             # TODO: This is probably wrong/not needed on macOS
             if not isMacOS():
-                yield makeIncludedGeneratedDataFile(
+                yield self.makeIncludedGeneratedDataFile(
                     data="""\
 [Paths]
 Prefix = .
@@ -668,7 +676,7 @@ Prefix = .
             resources_dir = self._getResourcesPath()
 
             for filename, filename_relative in listDir(resources_dir):
-                yield makeIncludedDataFile(
+                yield self.makeIncludedDataFile(
                     source_path=filename,
                     dest_path=os.path.join(
                         self._getResourcesTargetDir(), filename_relative
@@ -683,10 +691,11 @@ Prefix = .
                     filename_relative = os.path.relpath(filename, translations_path)
                     dest_path = self._getTranslationsTargetDir()
 
-                    yield makeIncludedDataFile(
+                    yield self.makeIncludedDataFile(
                         source_path=filename,
                         dest_path=os.path.join(dest_path, filename_relative),
                         reason="Qt translation",
+                        tags="translation",
                     )
 
     def getExtraDlls(self, module):
@@ -794,8 +803,8 @@ Prefix = .
                         dest_path=os.path.basename(dll_path),
                         package_name=full_name,
                     )
-        elif self.isQtWebEngineModule(full_name) and not self.webengine_done_binaries:
-            self.webengine_done_binaries = True  # prevent multiple copies
+        elif self.isQtWebEngineModule(full_name) and not self.web_engine_done_binaries:
+            self.web_engine_done_binaries = True  # prevent multiple copies
             self.info("Including QtWebEngine executable.")
 
             qt_web_engine_dir = self._getLibraryExecutablePath()
@@ -845,7 +854,7 @@ Prefix = .
                         if os.path.basename(sub_dll_filename).startswith(badword):
                             yield sub_dll_filename
 
-    def onModuleEncounter(self, module_filename, module_name, module_kind):
+    def onModuleEncounter(self, module_name, module_filename, module_kind):
         top_package_name = module_name.getTopLevelPackageName()
 
         if isStandaloneMode():
@@ -977,16 +986,14 @@ The standard PySide2 is not supported before CPython <3.6. For full support: htt
             self, qt_plugins=qt_plugins, no_qt_translations=no_qt_translations
         )
 
-    def onModuleEncounter(self, module_filename, module_name, module_kind):
-        # Enforce recursion in to multiprocessing for accelerated mode, which
-        # would normally avoid this.
+    def onModuleEncounter(self, module_name, module_filename, module_kind):
         if module_name == self.binding_name and self._getNuitkaPatchLevel() < 1:
             return True, "Need to monkey patch PySide2 for abstract methods."
 
         return NuitkaPluginQtBindingsPluginBase.onModuleEncounter(
             self,
-            module_filename=module_filename,
             module_name=module_name,
+            module_filename=module_filename,
             module_kind=module_kind,
         )
 
@@ -1158,3 +1165,32 @@ class NuitkaPluginPyQt6Plugins(NuitkaPluginQtBindingsPluginBase):
             """\
 Support for PyQt6 is experimental, use PySide6 if you can."""
         )
+
+
+class NuitkaPluginNoQt(NuitkaPluginBase):
+    """This is a plugins for suppression of all Qt binding plugins."""
+
+    plugin_name = "no-qt"
+    plugin_desc = "Disable all Qt bindings for standalone mode."
+
+    warned_about = set()
+
+    def onModuleEncounter(self, module_name, module_filename, module_kind):
+        top_package_name = module_name.getTopLevelPackageName()
+
+        if isStandaloneMode():
+            if top_package_name in _qt_binding_names:
+                if top_package_name not in self.warned_about:
+                    self.info(
+                        """\
+Unwanted import of '%(unwanted)s' that is forbidden encountered, preventing
+its use. As a result an "ImportError" might be given at run time. Uninstall
+it for full compatible behavior with the uncompiled code to debug it."""
+                        % {
+                            "unwanted": top_package_name,
+                        }
+                    )
+
+                    self.warned_about.add(top_package_name)
+
+                return (False, "Not included due to all Qt bindings disallowed.")
