@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,13 +19,9 @@
 
 One for each type of call. """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeCheck,
-    ExpressionAttributeLookup,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.BuiltinDictNodes import ExpressionBuiltinDict
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
@@ -45,12 +41,17 @@ from nuitka.nodes.ConditionalNodes import (
     makeStatementConditional,
 )
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
-from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.DictionaryNodes import StatementDictOperationSetKeyValue
+from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTuple
+from nuitka.nodes.DictionaryNodes import (
+    ExpressionDictOperationIteritems,
+    StatementDictOperationSet,
+    StatementDictOperationSetKeyValue,
+)
 from nuitka.nodes.ExceptionNodes import (
     ExpressionBuiltinMakeException,
     StatementRaiseException,
 )
+from nuitka.nodes.FunctionAttributeNodes import ExpressionFunctionErrorStr
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionCall,
     ExpressionFunctionCreation,
@@ -67,6 +68,8 @@ from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
     ExpressionBuiltinType1,
 )
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import StatementReleaseVariable
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
@@ -95,7 +98,7 @@ from .TreeHelpers import (
 
 
 def orderArgs(*args):
-    if python_version >= 350:
+    if python_version >= 0x350:
 
         def weight(arg):
             result = args.index(arg)
@@ -113,7 +116,7 @@ def orderArgs(*args):
 
 
 def _makeNameAttributeLookup(node, attribute_name="__name__"):
-    return ExpressionAttributeLookup(
+    return makeExpressionAttributeLookup(
         expression=node, attribute_name=attribute_name, source_ref=internal_source_ref
     )
 
@@ -128,11 +131,11 @@ def getCallableNameDescBody():
     #
     # called_type = type(BuiltinFunctionType)
     #
-    # if ininstance( called, (FunctionType, MethodType, BuiltinFunctionType) ):
+    # if isinstance(called, (FunctionType, MethodType, BuiltinFunctionType)):
     #     return called.__name__
-    # elif python_version < 3 and isinstance(called, ClassType):
+    # elif python_version < 0x3 and isinstance(called, ClassType):
     #     return called_type.__name__ + " constructor"
-    # elif python_version < 3 and isinstance(called, InstanceType):
+    # elif python_version < 0x3 and isinstance(called, InstanceType):
     #     return called_type.__name__ + " instance"
     # else:
     #     return called_type.__name__ + " object"
@@ -152,28 +155,45 @@ def getCallableNameDescBody():
 
     called_variable = result.getVariableForAssignment(variable_name="called")
 
+    if python_version < 0x390:
+        function_name = makeBinaryOperationNode(
+            operator="Add",
+            left=_makeNameAttributeLookup(
+                node=ExpressionVariableRef(
+                    variable=called_variable, source_ref=internal_source_ref
+                ),
+                attribute_name="__name__",
+            ),
+            right=makeConstantRefNode(
+                constant="()",
+                source_ref=internal_source_ref,
+                user_provided=True,
+            ),
+            source_ref=internal_source_ref,
+        )
+    else:
+        # TODO: Make it usable for pre-Python 3.9 too.
+        function_name = ExpressionFunctionErrorStr(
+            value=ExpressionVariableRef(
+                variable=called_variable, source_ref=internal_source_ref
+            ),
+            source_ref=internal_source_ref,
+        )
+
     functions_case = makeStatementsSequenceFromStatement(
         statement=(
             StatementReturn(
-                expression=makeBinaryOperationNode(
-                    operator="Add",
-                    right=makeConstantRefNode(
-                        constant="()",
-                        source_ref=internal_source_ref,
-                        user_provided=True,
-                    ),
-                    left=_makeNameAttributeLookup(
-                        ExpressionVariableRef(
-                            variable=called_variable, source_ref=internal_source_ref
-                        )
-                    ),
-                    source_ref=internal_source_ref,
-                ),
+                expression=function_name,
                 source_ref=internal_source_ref,
             )
         )
     )
 
+    if python_version >= 0x390:
+        result.setChild("body", functions_case)
+
+        return result
+
     no_branch = StatementReturn(
         expression=makeBinaryOperationNode(
             operator="Add",
@@ -193,7 +213,7 @@ def getCallableNameDescBody():
         source_ref=internal_source_ref,
     )
 
-    if python_version < 300:
+    if python_version < 0x300:
         instance_case = StatementReturn(
             expression=makeBinaryOperationNode(
                 operator="Add",
@@ -263,20 +283,20 @@ def getCallableNameDescBody():
             source_ref=internal_source_ref,
         )
 
-    if python_version < 300:
+    if python_version < 0x300:
         normal_cases = ("function", "builtin_function_or_method", "instancemethod")
     else:
         normal_cases = ("function", "builtin_function_or_method")
 
-    result.setBody(
+    result.setChild(
+        "body",
         makeStatementsSequenceFromStatement(
             statement=makeStatementConditional(
                 condition=ExpressionBuiltinIsinstance(
                     instance=ExpressionVariableRef(
                         variable=called_variable, source_ref=internal_source_ref
                     ),
-                    # TODO: This ought to be a constant value.
-                    classes=ExpressionMakeTuple(
+                    classes=makeExpressionMakeTuple(
                         elements=tuple(
                             ExpressionBuiltinAnonymousRef(
                                 builtin_name=builtin_name,
@@ -292,7 +312,7 @@ def getCallableNameDescBody():
                 no_branch=no_branch,
                 source_ref=internal_source_ref,
             )
-        )
+        ),
     )
 
     return result
@@ -310,7 +330,7 @@ def makeStarListArgumentErrorRaise(called_variable, star_list_variable):
                         source_ref=internal_source_ref,
                         user_provided=True,
                     ),
-                    right=ExpressionMakeTuple(
+                    right=makeExpressionMakeTuple(
                         elements=(
                             ExpressionFunctionCall(
                                 function=ExpressionFunctionCreation(
@@ -356,7 +376,7 @@ def makeStarListArgumentErrorRaise(called_variable, star_list_variable):
 
 
 def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
-    if python_version >= 350:
+    if python_version >= 0x350:
         non_tuple_code = makeStatementConditional(
             condition=ExpressionConditionalOr(
                 left=ExpressionAttributeCheck(
@@ -375,7 +395,7 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
                 ),
                 source_ref=internal_source_ref,
             ),
-            yes_branch=StatementAssignmentVariable(
+            yes_branch=makeStatementAssignmentVariable(
                 variable=star_list_variable,
                 source=ExpressionBuiltinTuple(
                     value=ExpressionVariableRef(
@@ -392,7 +412,7 @@ def _makeStarListArgumentToTupleStatement(called_variable, star_list_variable):
         )
     else:
         non_tuple_code = makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=star_list_variable,
                 source=ExpressionBuiltinTuple(
                     value=ExpressionVariableRef(
@@ -441,7 +461,7 @@ def _makeRaiseExceptionMustBeMapping(called_variable, star_dict_variable):
                         source_ref=internal_source_ref,
                         user_provided=True,
                     ),
-                    right=ExpressionMakeTuple(
+                    right=makeExpressionMakeTuple(
                         elements=(
                             ExpressionFunctionCall(
                                 function=ExpressionFunctionCreation(
@@ -489,7 +509,7 @@ def _makeRaiseExceptionMustBeMapping(called_variable, star_dict_variable):
 def _makeIteratingLoopStatement(tmp_iter_variable, tmp_item_variable, statements):
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -506,7 +526,7 @@ def _makeIteratingLoopStatement(tmp_iter_variable, tmp_item_variable, statements
         *statements
     )
 
-    return StatementLoop(body=loop_body, source_ref=internal_source_ref)
+    return StatementLoop(loop_body=loop_body, source_ref=internal_source_ref)
 
 
 def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_variable):
@@ -519,7 +539,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
 
     loop_body = []
 
-    if python_version >= 380:
+    if python_version >= 0x380:
         loop_body.append(
             makeStatementConditional(
                 condition=ExpressionComparisonIn(
@@ -540,14 +560,14 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
         )
 
     loop_body.append(
-        StatementAssignmentSubscript(
-            expression=ExpressionTempVariableRef(
+        StatementDictOperationSet(
+            dict_arg=ExpressionTempVariableRef(
                 variable=tmp_dict_variable, source_ref=internal_source_ref
             ),
-            subscript=ExpressionTempVariableRef(
+            key=ExpressionTempVariableRef(
                 variable=tmp_key_variable, source_ref=internal_source_ref
             ),
-            source=ExpressionSubscriptLookup(
+            value=ExpressionSubscriptLookup(
                 expression=ExpressionVariableRef(
                     variable=star_dict_variable, source_ref=internal_source_ref
                 ),
@@ -562,7 +582,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
 
     mapping_case = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_keys_variable,
                 source=makeCallNode(
                     _makeNameAttributeLookup(
@@ -581,7 +601,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionTempVariableRef(
@@ -591,7 +611,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_dict_variable,
             source=makeConstantRefNode(
                 constant={}, source_ref=internal_source_ref, user_provided=True
@@ -603,7 +623,7 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
             tmp_item_variable=tmp_key_variable,
             statements=loop_body,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=star_dict_variable,
             source=ExpressionTempVariableRef(
                 variable=tmp_dict_variable, source_ref=internal_source_ref
@@ -651,39 +671,45 @@ def _makeStarDictArgumentToDictStatement(result, called_variable, star_dict_vari
 
 
 def _makeRaiseNoStringItem(called_variable):
+    if python_version < 0x390:
+        raise_arg = makeBinaryOperationNode(
+            operator="Mod",
+            left=makeConstantRefNode(
+                constant="%s keywords must be strings",
+                source_ref=internal_source_ref,
+                user_provided=True,
+            ),
+            right=ExpressionFunctionCall(
+                function=ExpressionFunctionCreation(
+                    function_ref=ExpressionFunctionRef(
+                        function_body=getCallableNameDescBody(),
+                        source_ref=internal_source_ref,
+                    ),
+                    defaults=(),
+                    kw_defaults=None,
+                    annotations=None,
+                    source_ref=internal_source_ref,
+                ),
+                values=(
+                    ExpressionVariableRef(
+                        variable=called_variable, source_ref=internal_source_ref
+                    ),
+                ),
+                source_ref=internal_source_ref,
+            ),
+            source_ref=internal_source_ref,
+        )
+    else:
+        raise_arg = makeConstantRefNode(
+            constant="keywords must be strings",
+            source_ref=internal_source_ref,
+            user_provided=True,
+        )
+
     return StatementRaiseException(
         exception_type=ExpressionBuiltinMakeException(
             exception_name="TypeError",
-            args=(
-                makeBinaryOperationNode(
-                    operator="Mod",
-                    left=makeConstantRefNode(
-                        constant="""\
-%s keywords must be strings""",
-                        source_ref=internal_source_ref,
-                        user_provided=True,
-                    ),
-                    right=ExpressionFunctionCall(
-                        function=ExpressionFunctionCreation(
-                            function_ref=ExpressionFunctionRef(
-                                function_body=getCallableNameDescBody(),
-                                source_ref=internal_source_ref,
-                            ),
-                            defaults=(),
-                            kw_defaults=None,
-                            annotations=None,
-                            source_ref=internal_source_ref,
-                        ),
-                        values=(
-                            ExpressionVariableRef(
-                                variable=called_variable, source_ref=internal_source_ref
-                            ),
-                        ),
-                        source_ref=internal_source_ref,
-                    ),
-                    source_ref=internal_source_ref,
-                ),
-            ),
+            args=(raise_arg,),
             source_ref=internal_source_ref,
         ),
         exception_value=None,
@@ -706,7 +732,7 @@ def _makeRaiseDuplicationItem(called_variable, tmp_key_variable):
                         source_ref=internal_source_ref,
                         user_provided=True,
                     ),
-                    right=ExpressionMakeTuple(
+                    right=makeExpressionMakeTuple(
                         elements=(
                             ExpressionFunctionCall(
                                 function=ExpressionFunctionCreation(
@@ -749,18 +775,14 @@ def _makeRaiseDuplicationItem(called_variable, tmp_key_variable):
 def _makeStarDictArgumentMergeToKwStatement(
     result, called_variable, kw_variable, star_dict_variable
 ):
-    # This is plain terribly complex, pylint: disable=too-many-locals
+    # This is plain terribly complex
     temp_scope = result.allocateTempScope("dict")
 
-    tmp_dict_variable = result.allocateTempVariable(temp_scope, "dict")
     tmp_iter_variable = result.allocateTempVariable(temp_scope, "iter")
     tmp_keys_variable = result.allocateTempVariable(temp_scope, "keys")
     tmp_key_variable = result.allocateTempVariable(temp_scope, "key_xxx")
 
     final = [
-        StatementReleaseVariable(
-            variable=tmp_dict_variable, source_ref=internal_source_ref
-        ),
         StatementReleaseVariable(
             variable=tmp_iter_variable, source_ref=internal_source_ref
         ),
@@ -790,7 +812,7 @@ def _makeStarDictArgumentMergeToKwStatement(
             source_ref=internal_source_ref,
         ),
         StatementAssignmentSubscript(
-            expression=ExpressionVariableRef(
+            subscribed=ExpressionVariableRef(
                 variable=kw_variable, source_ref=internal_source_ref
             ),
             subscript=ExpressionTempVariableRef(
@@ -811,7 +833,7 @@ def _makeStarDictArgumentMergeToKwStatement(
 
     mapping_case = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_keys_variable,
                 source=makeCallNode(
                     _makeNameAttributeLookup(
@@ -830,7 +852,7 @@ def _makeStarDictArgumentMergeToKwStatement(
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionTempVariableRef(
@@ -840,13 +862,6 @@ def _makeStarDictArgumentMergeToKwStatement(
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
-            variable=tmp_dict_variable,
-            source=makeConstantRefNode(
-                constant={}, source_ref=internal_source_ref, user_provided=True
-            ),
-            source_ref=internal_source_ref,
-        ),
         _makeIteratingLoopStatement(
             tmp_iter_variable=tmp_iter_variable,
             tmp_item_variable=tmp_key_variable,
@@ -873,7 +888,7 @@ def _makeStarDictArgumentMergeToKwStatement(
     )
 
     dict_loop_body = (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_key_variable,
             source=ExpressionSubscriptLookup(
                 expression=ExpressionTempVariableRef(
@@ -903,7 +918,7 @@ def _makeStarDictArgumentMergeToKwStatement(
             source_ref=internal_source_ref,
         ),
         StatementAssignmentSubscript(
-            expression=ExpressionVariableRef(
+            subscribed=ExpressionVariableRef(
                 variable=kw_variable, source_ref=internal_source_ref
             ),
             subscript=ExpressionTempVariableRef(
@@ -923,7 +938,7 @@ def _makeStarDictArgumentMergeToKwStatement(
     )
 
     dict_case = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=kw_variable,
             source=ExpressionBuiltinDict(
                 pos_arg=ExpressionVariableRef(
@@ -934,17 +949,14 @@ def _makeStarDictArgumentMergeToKwStatement(
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
-                value=makeCallNode(
-                    _makeNameAttributeLookup(
-                        ExpressionVariableRef(
-                            variable=star_dict_variable, source_ref=internal_source_ref
-                        ),
-                        attribute_name="iteritems" if python_version < 300 else "items",
+                value=ExpressionDictOperationIteritems(
+                    dict_arg=ExpressionVariableRef(
+                        variable=star_dict_variable, source_ref=internal_source_ref
                     ),
-                    internal_source_ref,
+                    source_ref=internal_source_ref,
                 ),
                 source_ref=internal_source_ref,
             ),
@@ -1047,7 +1059,7 @@ def getFunctionCallHelperStarList():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1116,7 +1128,7 @@ def getFunctionCallHelperKeywordsStarList():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1190,7 +1202,7 @@ def getFunctionCallHelperPosStarList():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1267,7 +1279,7 @@ def getFunctionCallHelperPosKeywordsStarList():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1346,7 +1358,7 @@ def getFunctionCallHelperStarDict():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1430,7 +1442,7 @@ def getFunctionCallHelperPosStarDict():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1538,7 +1550,7 @@ def getFunctionCallHelperKeywordsStarDict():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1650,7 +1662,7 @@ def getFunctionCallHelperPosKeywordsStarDict():
         ),
     )
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1745,7 +1757,7 @@ def getFunctionCallHelperStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1789,7 +1801,7 @@ def getFunctionCallHelperPosStarListStarDict():
         star_arg_dict_variable=star_arg_dict_variable,
     )
 
-    if python_version >= 360:
+    if python_version >= 0x360:
         statements.reverse()
 
     statements.append(
@@ -1819,7 +1831,7 @@ def getFunctionCallHelperPosStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1883,7 +1895,7 @@ def getFunctionCallHelperKeywordsStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -1931,7 +1943,7 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
         star_arg_dict_variable=star_arg_dict_variable,
     )
 
-    if python_version >= 360:
+    if python_version >= 0x360:
         statements.reverse()
 
     statements.append(
@@ -1961,7 +1973,7 @@ def getFunctionCallHelperPosKeywordsStarListStarDict():
 
     body = makeStatementsSequenceFromStatements(*statements)
 
-    result.setBody(body)
+    result.setChild("body", body)
 
     return result
 
@@ -2051,7 +2063,7 @@ def getFunctionCallHelperDictionaryUnpacking():
     loop_body = (
         makeTryExceptSingleHandlerNode(
             tried=makeStatementsSequenceFromStatements(
-                StatementAssignmentVariable(
+                makeStatementAssignmentVariable(
                     variable=tmp_iter2_variable,
                     source=ExpressionBuiltinIter1(
                         value=makeCallNode(
@@ -2101,7 +2113,7 @@ def getFunctionCallHelperDictionaryUnpacking():
     )
 
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -2111,7 +2123,7 @@ def getFunctionCallHelperDictionaryUnpacking():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant={}, source_ref=internal_source_ref),
             source_ref=internal_source_ref,
@@ -2129,15 +2141,15 @@ def getFunctionCallHelperDictionaryUnpacking():
         ),
     )
 
-    result.setBody(
-        makeStatementsSequenceFromStatement(
-            makeTryFinallyStatement(
-                provider=result,
-                tried=tried,
-                final=final,
-                source_ref=internal_source_ref,
-            )
+    body = makeStatementsSequenceFromStatement(
+        makeTryFinallyStatement(
+            provider=result,
+            tried=tried,
+            final=final,
+            source_ref=internal_source_ref,
         )
     )
 
+    result.setChild("body", body)
+
     return result
