@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,26 +17,30 @@
 #
 """ Reformulation of lambda expressions.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
-from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
+from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNot
 from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantNoneRef
-from nuitka.nodes.FrameNodes import StatementsFrameFunction, StatementsFrameGenerator
-from nuitka.nodes.FunctionNodes import ExpressionFunctionCreation, ExpressionFunctionRef
+from nuitka.nodes.FrameNodes import (
+    StatementsFrameFunction,
+    StatementsFrameGenerator,
+)
+from nuitka.nodes.FunctionNodes import (
+    ExpressionFunctionCreation,
+    ExpressionFunctionRef,
+)
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
     ExpressionMakeGeneratorObject,
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableDelNodes import StatementReleaseVariable
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.nodes.YieldNodes import ExpressionYield
 from nuitka.PythonVersions import python_version
@@ -81,13 +85,15 @@ def buildLambdaNode(provider, node, source_ref):
             provider=function_body,
             name="<lambda>",
             code_object=code_object,
-            flags=set(),
+            flags=None,
+            auto_release=None,
             source_ref=source_ref,
         )
         code_body.qualname_provider = provider
 
     if function_kind == "Generator":
-        function_body.setBody(
+        function_body.setChild(
+            "body",
             makeStatementsSequenceFromStatement(
                 statement=StatementReturn(
                     expression=ExpressionMakeGeneratorObject(
@@ -98,7 +104,7 @@ def buildLambdaNode(provider, node, source_ref):
                     ),
                     source_ref=source_ref,
                 )
-            )
+            ),
         )
 
     defaults = buildNodeList(provider, node.args.defaults, source_ref)
@@ -109,17 +115,17 @@ def buildLambdaNode(provider, node, source_ref):
     body = buildNode(provider=code_body, node=node.body, source_ref=source_ref)
 
     if function_kind == "Generator":
-        if python_version < 270:
+        if python_version < 0x270:
             tmp_return_value = code_body.allocateTempVariable(
                 temp_scope=None, name="yield_return"
             )
 
             statements = (
-                StatementAssignmentVariable(
+                makeStatementAssignmentVariable(
                     variable=tmp_return_value, source=body, source_ref=source_ref
                 ),
                 makeStatementConditional(
-                    condition=ExpressionComparisonIsNOT(
+                    condition=ExpressionComparisonIsNot(
                         left=ExpressionTempVariableRef(
                             variable=tmp_return_value, source_ref=source_ref
                         ),
@@ -165,7 +171,7 @@ def buildLambdaNode(provider, node, source_ref):
 
     body = makeStatementsSequenceFromStatement(statement=body)
 
-    code_body.setBody(body)
+    code_body.setChild("body", body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
 
